"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.raw = void 0;
const file_loader_1 = __importDefault(require("file-loader"));
const imagemin_1 = __importDefault(require("imagemin"));
const schema_utils_1 = require("schema-utils");
const sharp_1 = __importDefault(require("sharp"));
const webpack_loader_util_1 = require("@calvin-l/webpack-loader-util");
const getFormat_1 = __importDefault(require("./helpers/getFormat"));
const getImageminPlugins_1 = __importDefault(require("./helpers/getImageminPlugins"));
const normalizeImageminOption_1 = __importDefault(require("./helpers/normalizeImageminOption"));
const replaceExtension_1 = __importDefault(require("./helpers/replaceExtension"));
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function default_1(content, sourceMap) {
    const callback = this.async();
    const defaultOptions = {
        format: getFormat_1.default(this.resourcePath),
        scaleUp: false,
        sharpOptions: {
            png: { quality: 100 },
            jpeg: { quality: 100 },
            webp: { quality: 100 },
            tiff: { quality: 100 },
        },
        imageminOptions: {
            png: {
                name: "imagemin-optipng",
                options: { interlaced: true, optimizationLevel: 7 },
            },
            jpeg: {
                name: "imagemin-mozjpeg",
                options: { quality: 80 },
            },
            webp: {
                name: "imagemin-webp",
                options: { quality: 75 },
            },
        },
    };
    const rawOptions = webpack_loader_util_1.getOptions(this, true, true);
    const options = {
        ...defaultOptions,
        sharpOptions: {
            ...defaultOptions.sharpOptions,
            ...rawOptions.sharpOptions,
        },
        imageminOptions: {
            ...defaultOptions.imageminOptions,
            ...rawOptions.imageminOptions,
        },
        ...rawOptions,
    };
    schema_utils_1.validate(options_json_1.default, options, {
        name: "Image Resize Loader",
        baseDataPath: "options",
    });
    processImage(content, options)
        .then((result) => {
        const fileLoaderContext = {
            ...this,
            resourcePath: replaceExtension_1.default(this.resourcePath, options.format),
            query: options.fileLoaderOptions,
        };
        const fileLoaderResult = file_loader_1.default.call(fileLoaderContext, result, sourceMap);
        callback(null, fileLoaderResult);
    })
        .catch((e) => {
        throw e;
    });
}
exports.default = default_1;
async function processImage(content, { width, height, fit, position, background, scale, format, quality, scaleUp, sharpOptions, imageminOptions, }) {
    let sharpImage = sharp_1.default(Buffer.from(content));
    const { height: imageHeight, width: imageWidth, } = await sharpImage.metadata();
    const normalizedImageHeight = imageHeight !== null && imageHeight !== void 0 ? imageHeight : Number.MAX_VALUE;
    const normalizedImageWidth = imageWidth !== null && imageWidth !== void 0 ? imageWidth : Number.MAX_VALUE;
    const normalizedResultHeight = height !== null && height !== void 0 ? height : 0;
    const normalizedResultWidth = width !== null && width !== void 0 ? width : 0;
    if ((width || height) &&
        (scaleUp ||
            (normalizedResultHeight <= normalizedImageHeight &&
                normalizedResultWidth <= normalizedImageWidth)))
        sharpImage = sharpImage.resize({
            width,
            height,
            fit,
            position,
            background,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    else if (scale && (scaleUp || scale <= 1)) {
        sharpImage = sharpImage.resize({
            width: Math.round(normalizedImageWidth * scale),
            fit,
            position,
            background,
            ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions.resize,
        });
    }
    sharpImage = sharpImage[format]({
        ...sharpOptions === null || sharpOptions === void 0 ? void 0 : sharpOptions[format],
    });
    const normalizedImageminOptions = normalizeImageminOption_1.default(imageminOptions === null || imageminOptions === void 0 ? void 0 : imageminOptions[format]);
    const imageminPlugins = getImageminPlugins_1.default(normalizedImageminOptions, {
        ...(quality ? { quality } : {}),
    });
    const sharpImageOutput = await sharpImage.toBuffer();
    if (imageminPlugins === undefined)
        return sharpImageOutput;
    const imageminOutput = await imagemin_1.default.buffer(sharpImageOutput, {
        plugins: imageminPlugins,
    });
    return imageminOutput;
}
//# sourceMappingURL=index.js.map